import datetime
import logging

from elasticsearch.helpers import scan

from grassat.database.base import EsBase


class FieldsDatabase(EsBase):
    def __init__(self, environment):
        super(FieldsDatabase, self).__init__()
        self.environment = environment
        self.index = 'databio-fields' if environment == 'prod' else 'databio-fields-dev'
        self._log = logging.getLogger("fieldsdb")

    def get_field_by_id(self, id) -> dict:
        """
        Retrieve the WatchItGrow field object from the database
        :param id: ID of the field to lookup
        :return:
        """
        self._log.debug(f'Looking up field with id {id}')
        result = self.es.get(index=self.index, doc_type=self.doc_type, id=id)
        if '_source' in result:
            return result['_source']
        raise Exception(f'Could not find field {id}')

    def update_field_warnings(self, id: str, warnings: list):
        """
        Update the warnings of an existing field through a partial update
        :param id: ID of the field to update
        :param warnings:  A list of warnings to attach to the field
        :return:
        """
        self._log.debug(f'Updating field warnings ({len(warnings)}) for field {id}')
        self.es.update(index=self.index, doc_type=self.doc_type, id=id, body={'doc': {
            'warnings': warnings
        }})

    def get_active_fields(self, type_varieties: list) -> list:
        """
        Fetch all active fields from the WIG database and filter all fields based on the croptype
        :param croptypes-variety combination:
        :return:
        """
        uniqueCropTypes = []
        for x in type_varieties:
            if x.split('-')[0] not in uniqueCropTypes:
                uniqueCropTypes.append(x.split('-')[0])
        fields = []
        for type in uniqueCropTypes:
            varieties = []
            for x in type_varieties:
                if x.split('-')[0] == type:
                    varieties.append(x.split('-')[1])
            current_date = datetime.datetime.now().date()
            query = {
                'query': {
                    'bool': {
                        'must': [
                            {
                                'range': {
                                    'metadata.startDate': {
                                        'lte': str(current_date),
                                    }
                                }
                            },
                            {
                                'range': {
                                    'metadata.endDate': {
                                        'gte': str(current_date),
                                    }
                                }
                            },
                            {
                                "term": {
                                    "metadata.country": "Belgium"
                                }
                            }
                        ],
                        'filter': [
                            {
                                'terms': {
                                    'cropFenology.variety': varieties
                                }
                            },
                            {
                                'terms': {
                                    'cropFenology.cropType': [type]
                                }
                            },
                            {
					            "bool": {
						            "should": [
							            {
								            "bool": {
									            "must_not": {
										            "exists": {
											            "field": "harvest.date"
										            }
									            }
								            }
							            },
							            {
								            "range": {
									            "harvest.date": {
										            "gte": str(current_date)
									                }
								            }
							            }
						            ]
					            }           
				            }
                        ]
                    }
                }
            }
            results = scan(
                client=self.es, query=query,
                index=self.index,
                size=100,
                scroll='120s')

            fields.extend(results)

        return fields
