import datetime
import logging
from dateutil.parser import parse

import numpy as np
import pandas as pd
from shapely.geometry import shape
from grassat.database.base import EsBase
from grassat.model.field_data import *



class MeteoDatabase(EsBase):
    def __init__(self):
        super(MeteoDatabase, self).__init__()
        self.index = 'meteo-kmi-5km'
        self._log = logging.getLogger("meteodb")

    def get_grid_nr(self, geometry: dict) -> int:
        """
        Retrieve the grid number for a given geometry
        :param geometry: Geometry for which to retrieve the grid number
        :return: Number presenting the grid number
        """
        self._log.debug(f'Looking up grid number based on a given geometry')
        query = {
            'query': {
                'bool': {
                    'must': {
                        "match_all": {}
                    },
                    'filter': {
                        'geo_shape': {
                            'location': {
                                'shape': geometry,
                                'relation': 'intersects'
                            }
                        }
                    }
                }
            }
        }
        hits = self.es.search(index=self.index, body=query, size=1)["hits"]["hits"]
        if len(hits) == 0:
            raise Exception('Could not find grid number for geometry')

        return hits[0]['_source']['GRID_NO']

    def get_meteo(self, field_data: FieldData, size: int = 9999) -> list:
        """
        Retrieve the meteo information that intersects with a given geometry
        :param geometry: Geometry for which to retrieve the meteo information
        :param start: Start date of the meteo
        :param end: End date of the meteo
        :param size: Maximum hits to return
        :return: DataFrame containing the daily meteo information for the specified WatchItGrow field
        """
        self._log.debug(f'Looking up meteo based on a given geometry')
        geometry = field_data.geometry
        start = parse(field_data.startDate).replace(tzinfo=None)
        end = min([parse(field_data.endDate).replace(tzinfo=None), datetime.today()])
        
        query = {
            'query': {
                'bool': {
                    'must': {
                        "range": {
                            "DAY": {
                                "gte": start.strftime('%Y-%m-%d'),
                                "lte": end.strftime('%Y-%m-%d')
                            }
                        }
                    },
                    'filter': {
                        'geo_shape': {
                            'location': {
                                'shape': geometry,
                                'relation': 'intersects'
                            }
                        }
                    }
                }
            },
            'sort': [
                {
                    'DAY': {
                        'order': 'asc'
                    }
                }
            ]
        }

        hits = self.es.search(index=self.index, body=query, size=size)["hits"]["hits"]

        if len(hits) == 0:
            raise Exception('Could not find meteo information for geometry')
        
        # Track unique GRID_NO values and their locations such as {944: shape object, 945: shape object, 990: shape object}
        grid_geometries = {}
    
        # Extract unique grid shapes from hits
        for hit in hits:
            grid_no = hit['_source']['GRID_NO']
            if grid_no not in grid_geometries:
                grid_geometries[grid_no] = shape(hit['_source']['location'])  # Store the unique grid shape

        logging.info(f'Grid no and geometries: {grid_geometries}')

        # If there's only one grid, no need to calculate areas
        if not len(grid_geometries) > 1:
            return list(map(lambda x: self._parse_meteo_data(data=x['_source'], country=field_data.country, site=field_data.community), hits))
    
        # Convert the input geometry (field) to a Shapely shape
        field_shape = shape(geometry)
    
        # Find the grid with the largest intersection area
        max_intersect_grid_no = None
        max_intersection_area = 0
        for grid_no, grid_shape in grid_geometries.items():
            intersection_area = field_shape.intersection(grid_shape).area
            logging.info(f'Intersection area of grid {grid_no}: {intersection_area}')
            if intersection_area > max_intersection_area:
                max_intersection_area = intersection_area
                max_intersect_grid_no = grid_no
        logging.info(f'Grid with the largest intersection: {max_intersect_grid_no}, Area: {max_intersection_area}')

        # Filter hits to only include data from the grid with the largest intersection
        filtered_hits = [hit for hit in hits if hit['_source']['GRID_NO'] == max_intersect_grid_no]
 
        return list(map(lambda x: self._parse_meteo_data(data=x['_source'], country=field_data.country, site=field_data.community), filtered_hits))

    def _parse_meteo_data(self, data: dict, country: str, site: str) -> dict:
        """
        Transform the meteo data to match the required output format.
        :param data: Meteo data from ElasticSearch
        :return: Dictionary with meteo information formatted for Grassat
        """
        # print("meteo data CALCULATED_RADIATION: ", data['CALCULATED_RADIATION'])
        # print("meteo data VAPOUR_PRESSURE: ", data['VAPOUR_PRESSURE'])
        # print("meteo data from db: ", data)

        date = datetime.strptime(data['DAY'].split('T')[0], '%Y-%m-%d')
    
        # Calculate Day of Year (DOY)
        doy = date.timetuple().tm_yday

        # Clip or scale data as per requirements
        tmin = self._clip(data['MINIMUM_TEMPERATURE'], -25, 40)  # Minimum temperature (°C)
        tmax = self._clip(data['MAXIMUM_TEMPERATURE'], -25, 40)  # Maximum temperature (°C)
        radiation = self._clip(data['CALCULATED_RADIATION'], 500, 40000)  # TODO: Check min, max value of radiation 
        # radiation = self._clip(data['CALCULATED_RADIATION']/ 1000, 5., 40.)  
        # radiation should be in kJ/m² is needed for radiation, in 'weerdata.csv', it looks that Joules per square meter (J/m²) is used instead of kJ/m²
        vapour_pressure = self._clip(data['VAPOUR_PRESSURE'] / 10, 0, 4) # Converting from hectopascals (hPa) to pascal (kPa), 10 hPa = 1 kPa.
        #  min, max value of vapour_pressure and conversion, vapour pressure should be in kP, I 
        wind_speed = self._clip(data['WINDSPEED'], 0, 15)  # Wind speed in m/s
        rainfall = data['RAINFALL']  # Rainfall in mm

        # Format the output to match the required structure in weerdata.csv
        return {
            'Year': date.year,
            'DOY': doy,
            'RAD': radiation,
            'MINTMP': tmin,
            'MAXTMP': tmax,
            'VP': vapour_pressure,
            'WIND': wind_speed,
            'RAIN': rainfall,
            'Country': country,      
            'Site': site       
    }


    def _clip(self, value, low, high):
        """
        Clip a given value between a low and high boundary
        :param value: Value to clip
        :param low: Minimum value to apply
        :param high: Maximum value to apply
        :return: Clipped value
        """
        return max(low, min(value, high))