
from .database.meteo import MeteoDatabase
import pandas as pd
from typing import Dict
from datetime import datetime, timedelta
from grassat.soil import *
from grassat.model.field_data import *



def getMeteoData(field_data: FieldData) -> pd.DataFrame:
    meteo_db = MeteoDatabase()
    meteo = meteo_db.get_meteo(field_data)
    meteo_df = pd.DataFrame(meteo)
    return meteo_df

def getIrrigations(irrigations) -> pd.DataFrame:
    # Convert irrigations to match format of irrigatie.csv
    irrigations_data = []
    if irrigations:
        for irrigation in irrigations:
            # Add 4 hours to the irrigation date to avoid timezone issues
            date_obj = datetime.strptime(irrigation["date"], "%Y-%m-%dT%H:%M:%S.%fZ") + timedelta(hours=4)
            
            # Get doy (day of year) from date
            start_of_year = datetime(date_obj.year, 1, 1)
            doy = (date_obj - start_of_year).days + 1  # +1 since DOY starts at 1
        
            # Check unit and convert dose to mm if necessary
            if irrigation["unit"] == "mm":
                dose_mm = irrigation["dose"]
            elif irrigation["unit"] == "l/m2":
                dose_mm = irrigation["dose"]  # 1 l/m2 is equivalent to 1 mm
            else:
                print(f"Skipping irrigation entry with invalid unit: '{irrigation['unit']}'.")
                continue  # Skip entry

            irrigations_data.append({"Dates": doy, "mm": dose_mm})

    # Create and sort the irrigations DataFrame
    irrigations_df = pd.DataFrame(irrigations_data, columns=["Dates", "mm"]).sort_values(by="Dates")
    
    # Aggregate mm values by Dates
    aggregated_df = irrigations_df.groupby("Dates", as_index=False).sum()

    if len(irrigations_df) > 0:
        # Aggregate kgN_ha values by Dates
        aggregated_df = irrigations_df.groupby("Dates", as_index=False).sum()
        return aggregated_df
    else:
        return irrigations_df

def getFertilisations(fertilisations) -> pd.DataFrame:
    # Define conversion factors to kg/ha
    conversion_factors = {
        "KG_HA": 1,
        "M_3_HA": 1000,  # 1 cubic meter = 1000 kg
        "TON_HA": 1000,  # 1 ton = 1000 kg
        "L_HA": 1,       # 1 l (liter) = 1 kg
    }

    # Convert fertilisations to match the format of mest.csv
    fertilisations_data = []
    if fertilisations:
        for fertilisation in fertilisations:
            # Add 4 hours to the fertilisation date to avoid timezone issues
            date_obj = datetime.strptime(fertilisation["date"], "%Y-%m-%dT%H:%M:%S.%fZ") + timedelta(hours=4)
            
            # Get doy (day of year) from date
            start_of_year = datetime(date_obj.year, 1, 1)
            doy = (date_obj - start_of_year).days + 1  # +1 since DOY starts at 1
            
            dose = fertilisation["products"][0]["dose"]
            dose_unit = fertilisation["products"][0]["doseUnit"]
        
            # Convert dose to kg/ha based on unit
            if dose_unit in conversion_factors:
                dose_kg_ha = dose * conversion_factors[dose_unit]
            else:
                print(f"Skipping fertilisation entry with invalid or unconvertible unit: '{dose_unit}'.")
                continue  # Skip entry

            fertilisations_data.append({"Dates": doy, "kgN_ha": dose_kg_ha})

    # Create and sort the fertilisations DataFrame, or create empty DataFrame if no data
    fertilisations_df = pd.DataFrame(fertilisations_data, columns=["Dates", "kgN_ha"]).sort_values(by="Dates")

    if len(fertilisations_df) > 0:
        # Aggregate kgN_ha values by Dates
        aggregated_df = fertilisations_df.groupby("Dates", as_index=False).sum()
        return aggregated_df
    else:
        return fertilisations_df

def getHarvestDays(harvests) -> list:
    harvestDays = set()  # Ensure uniqueness and avoid same doy if multiple harvest at same day
    for harvest in harvests:
        # Add 4 hours to harvest date to avoid timezone issues
        date = datetime.fromisoformat(harvest['date'].replace('Z', '')) + timedelta(hours=4)
        # Calculate the day of the year
        doy = date.timetuple().tm_yday # DOY starts from 1 by default
        harvestDays.add(doy)
    return sorted(harvestDays)
    
def getSoilData(geometry:dict) -> pd.DataFrame:
    soil_data = get_field_soil_data(geometry=geometry)
    return soil_data

